exports.__esModule = true;
exports['default'] = run;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _colors = require('colors');

var _colors2 = _interopRequireDefault(_colors);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _replace = require('replace');

var _replace2 = _interopRequireDefault(_replace);

var _nodeFetch = require('node-fetch');

var _nodeFetch2 = _interopRequireDefault(_nodeFetch);

var _sharedConsole = require('../../shared/console');

var _colors3 = require('./colors');

var _colors4 = _interopRequireDefault(_colors3);

var p = _path2['default'].join;

function run(_ref) {
  var name = _ref.name;
  var use = _ref.use;
  var nocache = _ref.nocache;
  var debug = _ref.debug;

  try {
    var spinner;

    (function () {
      var start = function start() {
        makeFolder().then(nocache ? cloneDirectly : getScaffold).then(initGit).then(replaceGivenNameInApp).then(finish).then(function () {
          spinner.stop();
        })['catch'](function (err) {
          spinner.stop();
          console.log("\n", 'Error'.bold.red);
          console.log(err);
        });
      };

      var makeFolder = function makeFolder() {
        return new Promise(function (resolve, reject) {
          log('makeFolder', MOTION.dest);
          _fsExtra2['default'].mkdirs(MOTION.scaffoldDir, function (err) {
            _fsExtra2['default'].mkdirs(MOTION.dest, function (err) {
              if (err) reject(err);else resolve();
            });
          });
        });
      };

      var getScaffold = function getScaffold() {
        return new Promise(function (res, rej) {
          // check online
          promiseProcess('curl http://github.com', { msg: false }).then(function () {
            log('Online');
            next();
          })['catch'](function () {
            log('Not online');
            copyScaffold().then(function () {
              return res();
            });
          });

          var next = _fsExtra2['default'].mkdirs.bind(null, MOTION.scaffoldDir, function (err) {
            log('fs.mkdirs err', err);
            var isCloningExample = repo != scaffoldRepo;

            if (err || isCloningExample) return cloneDirectly().then(function () {
              return res();
            });

            return updateScaffoldCache()['catch'](cloneDirectly).then(function (doCopy) {
              log('doCopy?', doCopy);
              if (doCopy) return copyScaffold();else return cloneDirectly();
            }).then(function () {
              // continue
              log('CONTINUE FROM getScaffold');
              res();
            });
          });
        });
      };

      var updateScaffoldCache = function updateScaffoldCache() {
        log('Looking for updated scaffold in %s', MOTION.scaffoldRepo);

        return new Promise(function (resolve, reject) {
          checkNewScaffold(function (needsNew) {
            log('checkNewScaffold needsNew?', needsNew);
            if (!needsNew) return resolve(true);

            // remove old scaffold
            _fsExtra2['default'].remove(MOTION.scaffoldDir, function (err) {
              log('fs.remove err?', err);
              if (err) return resolve(false);

              // clone new scaffold
              promiseProcess(gitClone(MOTION.scaffoldDir), { msg: false }).then(function () {
                return copyLatestSHA(MOTION.scaffoldDir);
              }).then(function () {
                return deleteGitFolder(MOTION.scaffoldDir);
              }).then(function () {
                return resolve(true);
              });
            });
          });
        });
      };

      var checkNewScaffold = function checkNewScaffold(cb) {
        log('Check for new scaffold SHA in...', MOTION.scaffoldSHA);
        _fsExtra2['default'].readFile(MOTION.scaffoldSHA, function (err, data) {
          if (err) {
            log('Error reading scaffold file');
            return cb(true);
          }

          var local = data.toString().trim();
          log('Got ', local);

          _nodeFetch2['default']('https://api.github.com/repos/motionjs/scaffold/commits').then(function (res) {
            return res.json();
          }).then(function (commits) {
            var remote = commits[0].sha.trim();
            log('Latest is', remote);

            if (local != remote) {
              message('Fetching latest scaffold...');
              // new
              cb(true);
            } else {
              // no new
              cb(false);
            }
          })['catch'](function (err) {
            cb(true);
          });
        });
      };

      var deleteGitFolder = function deleteGitFolder(dir) {
        return new Promise(function (resolve, reject) {
          log('Remove .git folder');
          // delete git dir
          _fsExtra2['default'].remove(p(dir, '/.git'), function (err) {
            log('after remove .git err?:', err);
            if (err) return reject(err);
            resolve();
          });
        });
      };

      var copyLatestSHA = function copyLatestSHA(dir) {
        return new Promise(function (res, rej) {
          log('copyLatestSHA');
          // copy latest SHA into folder
          var head = p(dir, '.git', 'refs', 'heads', 'master');
          log('Copy new SHA', head, MOTION.scaffoldSHA);

          try {
            var sha = _fsExtra2['default'].readFileSync(head);
            _fsExtra2['default'].writeFileSync(MOTION.scaffoldSHA, sha);
            res();
          } catch (e) {
            log('error with copyLatestSHA', e.stack);
            rej();
          }
        });
      };

      var copyScaffold = function copyScaffold() {
        log('Copy new scaffold', MOTION.scaffoldDir, MOTION.dest);
        return new Promise(function (res, rej) {
          _fsExtra2['default'].copy(MOTION.scaffoldDir, MOTION.dest, function (err) {
            if (err) {
              console.log("Error, couldn't copy scaffold folder".red);
              console.log(MOTION.scaffoldDir, MOTION.dest);
              process.exit(1);
            }

            res();
          });
        });
      };

      var gitClone = function gitClone(dest) {
        return 'git clone --depth=1 ' + MOTION.scaffoldRepo + ' ' + dest;
      }

      // clone right into target new folder
      ;

      var cloneDirectly = function cloneDirectly() {
        log('Cloning directly', gitClone(MOTION.dest));
        return promiseProcess(gitClone(MOTION.dest), { msg: false }).then(function () {
          return deleteGitFolder(MOTION.dest);
        });
      };

      var initGit = function initGit() {
        log('Init new git');
        return promiseProcess('git init', { msg: false });
      };

      var replaceGivenNameInApp = function replaceGivenNameInApp() {
        message('Setting app name...');
        return new Promise(function (resolve, reject) {
          log('Updating app name to', name, 'in', MOTION.dest);
          _replace2['default']({
            regex: 'motion-scaffold',
            replacement: name,
            paths: [MOTION.dest],
            recursive: true,
            silent: true
          });

          // random color :)
          _replace2['default']({
            regex: 'lightsalmon',
            replacement: _colors4['default'](),
            paths: [p(MOTION.dest, 'main.js')],
            recursive: false,
            silent: true
          });

          resolve();
        });
      };

      var npmInstall = function npmInstall() {
        log('Running npm install');
        return promiseProcess('npm install', {
          msg: 'Running npm install...',
          dir: MOTION.dest + '/.motion'
        });
      };

      var tryLinkMotion = function tryLinkMotion() {
        log('Try to link local motion');
        return promiseProcess('npm link motionjs', {
          msg: false,
          dir: MOTION.dest + '/.motion'
        });
      };

      var promiseProcess = function promiseProcess(cmd, opts) {
        opts = opts || {};

        if (opts.msg !== false) message(opts.msg || cmd);

        return new Promise(function (resolve, reject) {
          process.chdir(opts.appDir || MOTION.dest);
          log(' $ ', cmd);
          exec(cmd, { uid: process.getuid() }, function () {
            for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
              args[_key] = arguments[_key];
            }

            handleChildProcess.apply(undefined, [resolve, reject].concat(args));
          });
        });
      };

      var handleChildProcess = function handleChildProcess(resolve, reject, error, stdout, stderr) {
        if (error) {
          log(stderr);
          return reject(error);
        }

        log(stdout);
        resolve();
      };

      var finish = function finish() {
        message('Done!');
        spinner.stop();
        console.log('  Done! ⇢'.green.bold, ' cd ' + name + '\n');
        process.exit();
      };

      var message = function message(str) {
        spinner.message(str);
      };

      var log = function log() {
        if (debug) console.log.apply(console, arguments);
      }

      // mask (1 = execute, 4 = read, 2 = write)
      ;

      var checkPermission = function checkPermission(file, mask, cb) {
        _fsExtra2['default'].stat(file, function (error, stats) {
          if (error) {
            cb(error, false);
          } else {
            cb(null, !!(mask & parseInt((stats.mode & parseInt("777", 8)).toString(8)[0])));
          }
        });
      };

      var getUserHome = function getUserHome() {
        return process.env[process.platform == 'win32' ? 'USERPROFILE' : 'HOME'];
      };

      // scaffolds as second argument
      // motion new app from/repo
      var scaffoldRepo = 'scaffold';
      var org = 'motion';
      var repo = scaffoldRepo;

      // scaffold
      if (use) {
        // third party scaffold
        if (use.indexOf('/') >= 0) {
          org = use.split('/')[0];
          repo = use.split('/')[1];
        }
        // official motion scaffold
        else {
            repo = 'scaffold-' + use;
          }
      }

      // test for weird names
      if (/[^a-zA-Z0-9\_\-\$\.]/g.test(name)) {
        console.error('Name can\'t contain special characters'.bold.red);
        process.exit(1);
      }

      var MOTION = {};
      MOTION.scaffoldDir = p(getUserHome(), '.motion', 'scaffold');
      MOTION.scaffoldRepo = 'git://github.com/' + org + '/' + repo;
      MOTION.scaffoldSHA = p(getUserHome(), '.motion', 'scaffoldSHA');
      MOTION.dest = p(process.cwd(), name);

      if (_fsExtra2['default'].existsSync(MOTION.dest)) {
        console.log("Error! Directory %s already exists\n".red, MOTION.dest);
      } else {
        console.log();
        spinner = new _sharedConsole.Spinner('Creating app...  ');
        spinner.start();
        start();
      }

      var exec = require('child_process').exec;

      ;
    })();
  } catch (e) {
    console.log(e.stack);
  }
}

module.exports = exports['default'];