exports.__esModule = true;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var _stream = require('stream');

var _vinyl = require('vinyl');

var _vinyl2 = _interopRequireDefault(_vinyl);

var _helpers = require('./helpers');

var _index = require('../index');

var _opts = require('../../opts');

var _opts2 = _interopRequireDefault(_opts);

var _cache = require('../../cache');

var _cache2 = _interopRequireDefault(_cache);

var _bridge = require('../../bridge');

var _bridge2 = _interopRequireDefault(_bridge);

var _libFns = require('../../lib/fns');

// time we wait for browser load before we just force push
var UPPER_WAIT_LIMIT = 1000;
var isFileType = function isFileType(_path, ext) {
  return _libFns.path.extname(_path) == '.' + ext;
};
var debug = _libFns.log.bind(null, { name: 'stream', icon: '🎏' });

// socket for sending files to browser from editor
// has a lock system that waits for browser to load before sending more

var SuperStream = (function () {
  function SuperStream() {
    var _this = this;

    _classCallCheck(this, SuperStream);

    this.basePath = _opts2['default']('appDir');
    this.motionPath = _opts2['default']('motionDir');
    this.relPath = function (p) {
      return _libFns.path.relative(_this.basePath, p);
    };
    this.internalTimeout;
    this.browserLoading = {};
    this.queue = {};
    this.stream = new _stream.Readable({ objectMode: true });
    this.stream._read = function (n) {};

    this.watchForBrowserLoading();

    // watch, throttle the stream a bit
    _bridge2['default'].onMessage('live:save', _libFns._.throttle(this.fileSend.bind(this), 22, { leading: true }));

    // reset loading on errors in pipeline
    _index.event('error', function (_ref) {
      var path = _ref.path;
      return _this.setBrowserLoading(_this.relPath(path), false);
    });
  }

  SuperStream.prototype.getStream = function getStream() {
    return this.stream;
  };

  SuperStream.prototype.watchForBrowserLoading = function watchForBrowserLoading() {
    var _this2 = this;

    _bridge2['default'].onMessage('script:load', function (_ref2) {
      var path = _ref2.path;
      return _this2.setBrowserLoading(path, true);
    });
    _bridge2['default'].onMessage('script:done', function (_ref3) {
      var path = _ref3.path;
      return _this2.setBrowserLoading(path, false);
    });
  };

  SuperStream.prototype.setBrowserLoading = function setBrowserLoading(path, isLoading) {
    this.browserLoading[path] = isLoading;
    debug('IN', 'browser', isLoading ? 'loading'.red : 'done'.green, path);
    if (!isLoading) this.loadWaiting(path);
  };

  SuperStream.prototype.loadWaiting = function loadWaiting(path) {
    var queued = this.queue[this.relPath(path)];
    if (queued) queued();
  };

  SuperStream.prototype.fileSend = function fileSend(_ref4) {
    var _this3 = this;

    var path = _ref4.path;
    var startTime = _ref4.startTime;
    var contents = _ref4.contents;

    // check if file actually in motion project
    if (!path || path.indexOf(this.basePath) !== 0 || this.relPath(path).indexOf('.motion') === 0 || !isFileType(path, 'js')) {
      debug('  file not streamable', path.indexOf(this.basePath) !== 0, this.relPath(path).indexOf('.motion') === 0, !isFileType(path, 'js'));
      return;
    }

    // write to stream
    var relative = this.relPath(path);
    var sendImmediate = _cache2['default'].isInternal(path);

    debug('SIN', relative);

    this.pushStreamRun(relative, function () {

      debug('SOUT', relative);
      _this3.queue[relative] = false;
      // we may get another stream in before browser even starts loading
      // setBrowserLoading(relative, true)
      var file = new _vinyl2['default'](_libFns.vinyl(_this3.basePath, path, new Buffer(contents)));

      var stackTime = [{
        name: 'fileSend',
        time: +Date.now() - startTime
      }];

      Object.assign(file, { startTime: startTime, stackTime: stackTime });

      _this3.stream.push(file);
    }, sendImmediate);
  };

  SuperStream.prototype.pushStreamRun = function pushStreamRun(relative, finish, sendImmediate) {
    var _this4 = this;

    // waiting for script load
    if (!this.browserLoading[relative] || sendImmediate) return finish();

    // only queue once
    if (this.queue[relative]) return;

    this.queue[relative] = finish;

    // ensure upper limit on wait
    setTimeout(function () {
      if (!_this4.queue[relative]) return;

      debug('upper limit! finish'.yellow);
      _this4.browserLoading[relative] = false;
      finish();
    }, UPPER_WAIT_LIMIT);
  };

  return SuperStream;
})();

exports['default'] = SuperStream;
module.exports = exports['default'];