exports.__esModule = true;
exports.extractAndAssign = extractAndAssign;
exports.dynamicStyleStatement = dynamicStyleStatement;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _state = require('../state');

var _state2 = _interopRequireDefault(_state);

var _helpers = require('./helpers');

var _hashSum = require('hash-sum');

var _hashSum2 = _interopRequireDefault(_hashSum);

// splits styles into static/dynamic pieces

function extractAndAssign(node, file) {
  // if array of objects
  if (_helpers.t.isArrayExpression(node.right)) {
    return getArrayStatics(node, file);
  }

  // extract statics, but return just dynamics
  if (_helpers.t.isObjectExpression(node.right)) {
    var _ret = (function () {
      var name = node.left.name;

      if (_state2['default'].viewStyleNames[name]) throw file.errorWithNode(node.left, 'Duplicate style: view ' + _state2['default'].inView + ' { ' + name + ' }');

      _state2['default'].viewStyleNames[name] = true;

      var _extractStatics = extractStatics(name, node.right, file);

      var statics = _extractStatics.statics;
      var dynamics = _extractStatics.dynamics;

      // sets dynamic keys for use in determining hot reload clear later
      var statKeys = _state2['default'].viewStaticStyleKeys;
      var dynKeys = _state2['default'].viewDynamicStyleKeys;

      statics.forEach(function (n) {
        return statKeys[n.key.name] = _helpers.nodeToStr(n.value);
      });
      dynamics.forEach(function (n) {
        if (!_helpers.t.isSpreadProperty(n)) dynKeys[n.key.name] = _helpers.nodeToStr(n.value);
      });

      var hasStatics = statics.length;
      var hasDynamics = dynamics.length;

      var result = [];

      // if no dynamics, leave empty
      if (!hasStatics && !hasDynamics) return {
          v: result
        };

      // hot reload uniq keys

      // keep statics hash inside view for child view styles (to trigger hot reloads)
      var isChildView = hasStatics && name[1] && name[1] == name[1].toUpperCase();
      var isChildViewClassed = hasStatics && _state2['default'].viewHasChildWithClass && name != '$';

      if (isChildView || isChildViewClassed) {
        var uniq = _hashSum2['default'](statKeys);
        result.push(_helpers.exprStatement(_helpers.t.literal(uniq)));
      }

      // if dynamic + static clash, put that inside view to trigger hot reloads
      if (hasStatics && !_helpers.options.production && dynKeys.length) {
        var uniq = '';
        Object.keys(dynKeys).forEach(function (key) {
          if (statKeys[key]) {
            uniq += _hashSum2['default'](statKeys[key] + dynKeys[key]) + _hashSum2['default'](key);
          }
        });
        result.push(_helpers.exprStatement(_helpers.t.literal(uniq)));
      }

      // return statement

      if (hasDynamics) {
        result.push(dynamicStyleStatement(node, dynamics));
      }

      return {
        v: result
      };
    })();

    if (typeof _ret === 'object') return _ret.v;
  } else {
    return styleAssign(node);
  }
}

// find statics/dynamics in object
function extractStatics(name, node, file) {
  var statics = [];
  var dynamics = [];

  _state2['default'].viewStyles[_state2['default'].inView] = _state2['default'].viewStyles[_state2['default'].inView] || {};
  _state2['default'].viewStyles[_state2['default'].inView][name] = [];

  var duplicate = {};

  for (var _iterator = node.properties, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
    var _ref;

    if (_isArray) {
      if (_i >= _iterator.length) break;
      _ref = _iterator[_i++];
    } else {
      _i = _iterator.next();
      if (_i.done) break;
      _ref = _i.value;
    }

    var prop = _ref;

    if (_helpers.t.isSpreadProperty(prop)) {
      // TODO: make work
      dynamics.push(prop);
      continue;
    }

    if (duplicate[prop.key.name]) throw file.errorWithNode(prop, 'Duplicate style property: view ' + _state2['default'].inView + ' { ' + name + ' = { ' + prop.key.name + ' } }');

    duplicate[prop.key.name] = true;

    if (isStatic(prop)) {
      _state2['default'].viewStyles[_state2['default'].inView][name].push(prop);
      statics.push(prop);
    } else {
      dynamics.push(prop);
    }
  }

  return { statics: statics, dynamics: dynamics };
}

// determine if property is static
function isStatic(prop) {
  var staticKey = _helpers.t.isIdentifier(prop.key);

  if (!staticKey) return false;

  var staticVal = _helpers.t.isLiteral(prop.value) || _helpers.t.isUnaryExpression(prop.value) && _helpers.t.isLiteral(prop.value.argument);

  if (staticVal) return true;

  // determine if array is fully static
  if (_helpers.t.isArrayExpression(prop.value)) {
    return prop.value.elements.reduce(function (acc, cur) {
      return acc = acc && _helpers.t.isLiteral(cur);
    }, true);
  }
}

// $["name"] = ...

function dynamicStyleStatement(node, dynamics) {
  return _helpers.exprStatement(styleAssign(node, _helpers.t.isArrayExpression(dynamics) ? dynamics : _helpers.t.objectExpression(dynamics)));
}

function styleLeft(node) {
  var prefix = '$';

  if (node.left.object) {
    return node.left;
  }

  var name = node.left.name.slice(1) || '$';
  return _helpers.t.identifier(prefix + '["' + name + '"]');
}

function styleAssign(node, _right) {
  var right = _right || node.right;

  var assignment = _helpers.t.assignmentExpression('=', styleLeft(node), styleFunction(right));

  assignment.isStyle = true;
  return assignment;

  // (_index) => {}
  function styleFunction(inner) {
    return _helpers.t.functionExpression(null, [_helpers.t.identifier('_'), _helpers.t.identifier('_index')], _helpers.t.blockStatement([_helpers.t.returnStatement(inner)]));
  }
}

// extract statics
function getArrayStatics(node, file) {
  var rightEls = [];
  var staticProps = [];

  var result = function result() {
    return dynamicStyleStatement(node, node.right);
  };

  for (var _iterator2 = node.right.elements, _isArray2 = Array.isArray(_iterator2), _i2 = 0, _iterator2 = _isArray2 ? _iterator2 : _iterator2[Symbol.iterator]();;) {
    var _ref2;

    if (_isArray2) {
      if (_i2 >= _iterator2.length) break;
      _ref2 = _iterator2[_i2++];
    } else {
      _i2 = _iterator2.next();
      if (_i2.done) break;
      _ref2 = _i2.value;
    }

    var el = _ref2;

    // bail out if they arent using just objects (ternery, variable)
    if (!_helpers.t.isObjectExpression(el)) {
      return result();
    }

    var extracted = extractStatics(node.left.name, el, file);
    if (!extracted) continue;

    var statics = extracted.statics;
    var dynamics = extracted.dynamics;

    if (statics.length) staticProps = staticProps.concat(statics);

    if (dynamics.length) {
      rightEls.push(_helpers.t.objectExpression(dynamics));
      continue;
    }
  }

  node.right.elements = rightEls;

  return result();
}