exports.__esModule = true;
exports['default'] = init;
exports.t = t;
exports.options = options;
exports.niceJSXAttributes = niceJSXAttributes;
exports.isUpperCase = isUpperCase;
exports.viewMainSelector = viewMainSelector;
exports.viewSelector = viewSelector;
exports.hasObjWithProp = hasObjWithProp;
exports.findObjectName = findObjectName;
exports.isInView = isInView;
exports.getRootTagName = getRootTagName;
exports.getTagName = getTagName;
exports.isMutativeArrayFunc = isMutativeArrayFunc;
exports.isObjectAssign = isObjectAssign;
exports.relativePath = relativePath;
exports.exprStatement = exprStatement;
exports.normalizeLocation = normalizeLocation;
exports.nodeToStr = nodeToStr;
exports.frozen = frozen;
exports.propChange = propChange;
exports.nodeToNameString = nodeToNameString;
exports.isJSXAttributeOfName = isJSXAttributeOfName;
exports.tracker = tracker;
exports.destructureTrackers = destructureTrackers;
exports.parentFunctionNode = parentFunctionNode;
exports.shouldStyleAsRoot = shouldStyleAsRoot;
exports.isViewState = isViewState;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _state = require('../state');

var _state2 = _interopRequireDefault(_state);

var T = undefined,
    options = undefined;

function init(_options, _t) {
  // redefine t??
  exports.t = t = t.t = _t;
  exports.options = exports.options = exports.options = options = _options;
}

function t() {}

function options() {}

var niceAttrs = {
  className: 'class',
  htmlFor: 'for',
  srcSet: 'srcset',
  noValidate: 'novalidate',
  autoPlay: 'autoplay',
  frameBorder: 'frameborder',
  allowFullScreen: 'allowfullscreen',
  tabIndex: 'tabindex'
};

function niceJSXAttributes(name) {
  for (var key in niceAttrs) {
    if (name == niceAttrs[key]) {
      return key;
    }
  }
  return name;
}

function isUpperCase(str) {
  return str.charAt(0) == str.charAt(0).toUpperCase();
}

function viewMainSelector(viewName, options) {
  var pre = options.selectorPrefix || '';
  return pre + '.View' + viewName;
}

function viewSelector(viewName, tag, options) {
  var pre = options.selectorPrefix || '';
  var selTag = tag + '.' + viewName;
  var selClass = '.' + viewName + '.' + tag;
  var selSelfClass = '.View' + viewName + '.' + tag;
  // const selChildClass = `.${viewName} > .${tag}` // for children views?
  return pre + selTag + ', ' + (pre + selClass) + ', ' + (pre + selSelfClass);
}

function hasObjWithProp(node, base, prop) {
  return node.left && node.left.object && node.left.object.name == base && (!prop || node.left.property && node.left.property.name == prop);
}

function findObjectName(_x2) {
  var _again = true;

  _function: while (_again) {
    var obj = _x2;
    _again = false;

    if (obj.name) return obj.name;
    if (obj.object) {
      _x2 = obj.object;
      _again = true;
      continue _function;
    }
  }
}

function isInView(scope) {
  return scope.hasBinding("view");
}

function getRootTagName() {
  if (!_state2['default'].viewRootNodes.length) return '';
  return getTagName(_state2['default'].viewRootNodes[0]);
}

function getTagName(JSXElement) {
  return JSXElement.openingElement.name.elements[0].value;
}

var mutativeFuncs = ['push', 'reverse', 'splice', 'shift', 'pop', 'unshift', 'sort'];

function isMutativeArrayFunc(node) {
  var name = node.callee && node.callee.property && node.callee.property.name;

  return name && mutativeFuncs.indexOf(name) >= 0;
}

function isObjectAssign(node) {
  if (!node.callee) return;

  var propName = node.callee.property && node.callee.property.name;
  var objName = node.callee.object && node.callee.object.name;

  return objName == 'Object' && propName == 'assign';
}

var idFn = function idFn(x) {
  return x;
};

exports.idFn = idFn;
// helpers

function relativePath(filename) {
  if (filename && options.basePath) {
    return _path2['default'].relative(options.basePath, filename);
  }
  return '';
}

function exprStatement(node) {
  return t.expressionStatement(node);
}

function normalizeLocation(location) {
  return [[location.start.line - 1, location.start.column], [location.end.line - 1, location.end.column]];
}

// return unique string based on value of node

function nodeToStr(node) {
  if (t.isMemberExpression(node)) return node.object.name + node.property.name;
  if (t.isArrayExpression(node)) return node.elements.reduce(function (acc, cur) {
    return acc + nodeToStr(cur);
  }, '');
  if (t.isObjectExpression(node)) {
    return node.properties.reduce(function (acc, cur) {
      return acc + typeof cur.key == 'object' && cur.key.name + nodeToStr(cur.value);
    }, '');
  }

  return node && node.value;
}

function frozen(node) {
  return t.callExpression(t.identifier('Object.freeze'), [node]);
}

function propChange(node) {
  return t.expressionStatement(t.callExpression(t.identifier('on.props'), [t.functionExpression(null, [], t.blockStatement(node.declarations.map(function (_ref) {
    var name = _ref.id.name;
    return t.expressionStatement(t.assignmentExpression('=', t.identifier(name), t.identifier('view.getProp(\'' + name + '\')')));
  })))]));
}

function nodeToNameString(node) {
  if (typeof node.name == 'string') return node.name;

  if (t.isJSXMemberExpression(node)) {
    var isNested = t.isJSXMemberExpression(node.object);
    return (isNested ? nodeToNameString(node.object) : '') + ((node.object.name || '') + '.' + node.property.name);
  }
}

function isJSXAttributeOfName(attr, name) {
  return attr.name == name;
}

function tracker(name) {
  var type = arguments.length <= 1 || arguments[1] === undefined ? 'dec' : arguments[1];

  return t.expressionStatement(t.callExpression(t.identifier('view.' + type), [t.literal(name), t.identifier(name)]));
}

function destructureTrackers(id, wrapType) {
  return id.properties.map(function (prop) {
    return tracker(prop.key.name, wrapType);
  });
}

function parentFunctionNode(scope) {
  if (t.isArrowFunctionExpression(scope.parentBlock.init)) return scope.parentBlock.init;

  var parentFunc = scope.path.findParent(function (p) {
    return p.isFunction();
  });
  return parentFunc && parentFunc.node;
}

function shouldStyleAsRoot() {
  var numRoots = _state2['default'].viewRootNodes.length;
  var result = numRoots == 0;
  if (numRoots == 1) {
    var hasRootProp = _state2['default'].viewRootNodes[0].openingElement.attributes.filter(function (x) {
      return x.name && x.name.name === 'root';
    }).length;

    result = hasRootProp || getRootTagName() == _state2['default'].inView.toLowerCase();
  }
  return result;
}

function isViewState(name, scope) {
  return _state2['default'].viewState[name] && !scope.hasOwnBinding(name);
}