exports.__esModule = true;
exports.updateState = updateState;
exports.wrapSetter = wrapSetter;
exports.wrapDeclarator = wrapDeclarator;
exports.wrapPropertyDeclarator = wrapPropertyDeclarator;
exports.getter = getter;
exports.viewGetter = viewGetter;
exports.wrapGetter = wrapGetter;
exports.stateTrack = stateTrack;

var _helpers = require('./helpers');

function updateState() {
  return _helpers.t.expressionStatement(_helpers.t.callExpression(_helpers.t.identifier('view.updateSoft'), []));
}

function wrapSetter(name, node, scope, postfix) {
  var method = arguments.length <= 4 || arguments[4] === undefined ? 'set' : arguments[4];

  if (node.hasSetter) return;
  if (scope.hasBinding('view')) {
    var args = [_helpers.t.literal(name), node];
    if (postfix) args = args.concat([postfix, _helpers.t.identifier('true')]);
    var expr = _helpers.t.callExpression(_helpers.t.identifier('view.' + method), args);
    node.hasSetter = true;

    // for later view.update() insertion
    var scopeBlockIsFunc = _helpers.t.isArrowFunctionExpression(scope.block) || _helpers.t.isFunctionExpression(scope.block) || _helpers.t.isFunctionDeclaration(scope.block);

    if (scopeBlockIsFunc) {
      scope.block.hasSetter = true;
    } else {
      // or find parent
      var parentFunc = _helpers.parentFunctionNode(scope);
      if (parentFunc) parentFunc.motionStateMutativeFunction = true;
    }

    return expr;
  }

  return node;
}

function wrapDeclarator(name, node, scope) {
  return wrapSetter(name, node, scope, false, 'dec');
}

function wrapPropertyDeclarator(name, node, scope) {
  return wrapSetter(name, node, scope, false, 'prop');
}

function getter(name, val) {
  for (var _len = arguments.length, args = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    args[_key - 2] = arguments[_key];
  }

  return _helpers.t.callExpression(_helpers.t.identifier('view.get'), [_helpers.t.literal(name), val].concat(args));
}

function viewGetter(name, val, scope, file) {
  var comesFromFile = file.scope.hasOwnBinding(val.name);

  if (comesFromFile) return getter(name, val, _helpers.t.literal('fromFile'));

  return getter(name, val);
}

function wrapGetter(node, scope, file) {
  if (node.hasGetter) return;
  if (scope.hasOwnBinding('view')) {
    if (node.left.object) return node;
    node.right = viewGetter(node.left.name, node.right, scope, file);
    node.hasGetter = true;
  }
  return node;
}

function stateTrack(node) {
  if (node.body.motionView) return;
  if (node.motionStateTracked) return;
  node.motionStateTracked = true;

  if (node.hasSetter || node.motionStateMutativeFunction) {
    if (_helpers.t.isArrowFunctionExpression(node) && _helpers.t.isCallExpression(node.body)) {
      node.body = wrapper(node.body);
    } else {
      node.body.body = wrapper(node.body.body);
    }
  }

  return node;

  function wrapper(body) {
    if (Array.isArray(body)) {
      body.push(updateState());
    } else {
      body = _helpers.t.blockStatement([_helpers.t.expressionStatement(body), updateState()]);
    }
    return body;
  }
}